(:~
 : This module creates an HTML serialization for the modern edition's forewords.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 :)

xquery version "3.1";

module namespace foreword="http://bdn-edition.de/ns/foreword";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

(:~
 : The main transformation function.
 :
 : @author Michelle Weidling
 : @param $nodes The modern foreword's TEI nodes
 : @param $note-map A map for tei:note's IDs and their md5 hashes
 : @return The processed node as an XHTML element
 :)
declare function foreword:transform($nodes as node()*, $note-map as map(*))
as item()*
{
  for $node in $nodes return
    typeswitch($node)
      case text() return
          $node

      case element(tei:div) return
          if ($node/@type = "contents"
          or $node/@subtype = "print") then
              ()
          else
              element xhtml:div {
                  if ($node/@type) then
                      attribute class {"foreword-" || $node/@type}
                  else
                      (),
                  foreword:transform($node/node(), $note-map)
              }

      case element(tei:head) return
          if ($node/@type
          or $node/parent::tei:div[@type]) then
              foreword:make-block-header($node, $note-map)
          else
              let $alignment-class :=
                  if ($node/parent::tei:div[not(@type)]) then
                      "centered"
                  else
                      "not-centered"
              let $level := string(count($node/ancestor::tei:div))
              let $id := generate-id($node)
              return
                  switch ($level)
                  case "1" return
                      foreword:make-block-header($node, $note-map)
                  case "2" return
                      <h3 class="{$alignment-class}" id="{$id}">{foreword:transform($node/node(), $note-map)}</h3>
                  case "3" return
                      <h4 class="{$alignment-class}" id="{$id}">{foreword:transform($node/node(), $note-map)}</h4>
                  case "4" return
                      <h5 class="{$alignment-class}" id="{$id}">{foreword:transform($node/node(), $note-map)}</h5>
                  default return
                      <div id="{$id}" class="low-level-heading {$alignment-class}">
                          {foreword:transform($node/node(), $note-map)}
                      </div>

      case element(tei:p) return
          element xhtml:p {
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:closer) return
          element xhtml:div {
              attribute class {$node/name()},
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:dateline) return
          element xhtml:span {
              attribute class {$node/name()},
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:signed) return
          element xhtml:span {
              attribute class {$node/name()},
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:hi) return
          if (not($node/@rend)) then
              element xhtml:em {
                  foreword:transform($node/node(), $note-map)
              }
          else
              element xhtml:sup {
                  foreword:transform($node/node(), $note-map)
              }

      case element(tei:divGen) return
          ()

      case element(tei:note) return
        element xhtml:span {
          attribute href {"#" || map:get($note-map, $node/@id)},
          attribute data-toggle {"modal"},
          attribute id {$node/@xml:id},
          <i class="far fa-comment"></i>
        }

      case element(tei:ref) return
      (
         $node/text(),
         element a
            {
                attribute class { "text-link" },
                attribute href { $node/@target },
                <i class="fas fa-external-link-alt"></i>
            }
      )

      case element(tei:list) return
          element xhtml:ul {
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:item) return
          element xhtml:li {
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:lb) return
          <br />

      case element(tei:table) return
          element xhtml:table {
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:row) return
          element xhtml:tr {
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:cell) return
          element xhtml:td {
              foreword:transform($node/node(), $note-map)
          }

      case element(tei:front) return
          (foreword:transform($node/node(), $note-map),
          for $note in $node//tei:note return
              <div class="modal fade" role="dialog" id="{map:get($note-map, $note/@id)}">
                  <div class="modal-dialog">
                      <div class="modal-content">
                          <div class="modal-header">
                              <button type="button" class="close" data-dismiss="modal">
                              × <!-- times -->
                              </button>
                          </div>
                          <div class="modal-body">
                              <p>{foreword:transform($note/node(), $note-map)}</p>
                          </div>
                      </div>
                  </div>
              </div>)

  default return
    foreword:transform($node/node(), $note-map)
};

(:~
 : An auxiliary function for creating block headers.
 :
 : @author Michelle Weidling
 : @param $node The current tei:head element
 : @param $note-map A map for tei:note's IDs and their md5 hashes
 : @return A block header
 :)
declare function foreword:make-block-header($node as element(tei:head), $note-map as map(*)) as element(xhtml:div)
{
    <div class="block-header {$node/@type}">
        <h2 id="{generate-id($node)}">
            <span class="title">
                {foreword:transform($node/node(), $note-map)}
            </span>
            <span class="decoration" />
            <span class="decoration" />
            <span class="decoration" />
        </h2>
    </div>
};
